# app.py
# Author: Labeller <https://github.com/mdbloice/Labeller>
from flask_bootstrap import Bootstrap
from flask import Flask, render_template, g, request, abort, jsonify
import os
import glob
import random
import sqlite3
import pickle

app = Flask(__name__)
Bootstrap(app)

# Read the config file which contains the class names.
# NOT CURRENTLY NEEDED
#with open('labeller.pkl', 'rb') as to_read:
#    class_names = pickle.load(to_read)

extensions = ["*.png", "*.PNG", "*.jpg", "*.JPG", "*.jpeg", "*.JPEG"]
images = []
for ext in extensions:
    images.extend(glob.glob(os.path.join('.', 'static', 'images', ext)))

print("Found %s images." % len(images))

# SQLite Database
DATABASE = os.path.join('.', 'db', 'labels.db')

def get_db():
    db = getattr(g, '_database', None)

    if db is None:
        db = g._database = sqlite3.connect(DATABASE)
    return db

@app.teardown_appcontext
def teardown_db(exception):
    db = getattr(g, '_database', None)

    if db is not None:
        db.close()

def query_db(query, args=(), one=False):
    cur = get_db().execute(query, args)
    rv = cur.fetchall()
    cur.close()
    return (rv[0] if rv else None) if one else rv

@app.route('/api/image', methods=['POST'])
def set_image():
    if not request.json or not 'tile' in request.json or not 'tag' in request.json:
        abort(400)

    q = "INSERT INTO labels (image, label, label_string) VALUES ('%s', %s)"  \
        % (request.json['tile'], request.json['tag'])

    con =  get_db()
    cursor = con.cursor()
    cursor.execute(q)
    con.commit()
    cursor.close()

    return jsonify({"Status": "OK"})

@app.route('/')
def index():
        image_count = len(images)
        r = random.randint(0, image_count-1)
        rand_image= images[r]

        return render_template(
        'index.html', images=images,
        rand_image=rand_image, image_count=image_count)

@app.route('/about.html')
def about():
    return render_template('about.html')

@app.route('/labels.html')
def labels():
    labels = query_db("SELECT * FROM labels")
    return render_template('labels.html', labels=labels)

#if __name__ == '__main__':
#    app.run(debug=False, use_reloader=False, port=5000)


# htmlElements.py
# Author: Marcus D. Bloice <https://github.com/mdbloice>
# Contains dynamic HTML elements that cannot be copied from the resources
# directory, where static HTML and CSS is stored.

class Button():
    def __init__(self, btn_id, btn_label) -> None:
        self.btn_label = btn_label
    def get_html(self):
        return '<a id="%s" class="btn btn-lg btn-default" role="button" style="width: 100px">%s</a>' % (self.btn_id, self.btn_label)


class KeyPressJS():
    def __init__(self, classes) -> None:
        self.classes = classes
    def get_html(self):

        # keyCode values for the following numbers:
        # 1 = 49
        # 2 = 50
        # 3 = 51
        # 4 = 52
        # 5 = 53
        # 6 = 54
        # 7 = 55
        # 8 = 56
        # 9 = 57
        # Therefore the class number + 48
        # This is only to be done for the classes 1-9

        js_elements = []

        l = len(self.classes) if len(self.classes) <= 9 else 10

        for i in range(l):

            js = """
            if (e.keyCode == %s) {
                console.log(
                    "Handler for keypress fired with keyCode: " + e.keyCode + " (%s)"
                );
            postToDB(%s);
            getNewImage();

            """ % ((i+1)+48, self.classes[i], i+1)

            js_elements.append(js)

        return """
        $(document).keypress(function (e) {

            %s

            }
        });
        """ % ' '.join(js_elements)


# Not currently used as we use the static navbar.html file
# from the package's resources directory.
class Navbar():
    def __init__(self) -> None:
        pass

    def get_html(self) -> str:

        navbar_contents = """
        {% block navbar %}
        <nav class="navbar navbar-default">
            <div class="container-fluid">
                <div class="navbar-header">
                <a class="navbar-brand" href="/"><span class="glyphicon glyphicon-tag"></span> Labeller</a>
                </div>
                <ul class="nav navbar-nav">
                <li class="{{ 'active' if active_page == 'index' else '' }}"><a href="/"><span class="glyphicon glyphicon-home"></span> Label Images</a></li>
                <li class="{{ 'active' if active_page == 'labels' else '' }}"><a href="labels.html"><span class="glyphicon glyphicon-oil"></span> Label Database</a></li>
                <li class="{{ 'active' if active_page == 'about' else '' }}"><a href="about.html"><span class="glyphicon glyphicon-question-sign"></span> About</a></li>
                </ul>
            </div>
        </nav>
        {% endblock %}
        """

        return navbar_contents


class Footer():
    def __init__(self, footerText='Created with <a href="https://github.com/mdbloice/Labeller" target="_blank"><span class="glyphicon glyphicon-tag"></span> Labeller</a>') -> None:
        self.footerText = footerText

    def get_html(self) -> str:
        return """
        <div class="container">
        <hr>
        <footer class="bg-light text-center text-lg-start" style="padding: 10px;">
            <div class="text-center p-3">
            %s
            </div>
        </footer>
        </div>
        """ % self.footerText


class Index():
    def __init__(self, class_names) -> None:
        self.class_names = class_names
        self.n_classes = len(class_names)

    def get_html(self):

        keyboard_shortcuts = []
        buttons = []
        post_functions = []

        for i in range(self.n_classes):
            keyboard_shortcuts.append("<li><kbd>%s</kbd> for <b>%s</b></li>" % (i+1, self.class_names[i]))
            buttons.append('<a id="%s" class="btn btn-lg btn-default" role="button">%s</a>' % (self.class_names[i], self.class_names[i]))
            post_functions.append('$("#%s").click(function () { postToDB(%s); getNewImage(); });' % (self.class_names[i], i))

        # Note: % symbols used by Jinja must be escaped as %% or Python
        # string replacements will not function correctly.
        return """
        {%% extends "bootstrap/base.html" %%} {%% set active_page = "index" %%} {%% block
        head %%} {{ super() }}
        <link
        rel="shortcut icon"
        href="{{ url_for('static', filename='favicon.ico') }}"
        />
        {%% endblock %%} {%% block scripts %%} {{ super() }}

        <script>
        const imageURL = "{{ url_for('set_image') }}";

        var l = JSON.parse('{{ images|tojson|safe }}');
        var count = l.length;
        //var l = JSON.parse("{{ images|tojson }}");
        var count = l.length;

        function getNewImage() {
            var randomImage = Math.floor(Math.random() * count);
            var displayNumber = randomImage + 1;
            document.getElementById("image").src = l[randomImage];
            document.getElementById("imageText").innerHTML =
            "Image " + displayNumber + " of " + count + " (" + l[randomImage] + ")";
        }

        function postToDB(taggedAs) {
            var currentImage = document.getElementById("image").src;
            var currentImage = currentImage.split("/static/")[1];

            const data = {
            image: currentImage,
            label: taggedAs
            };

            const dataJSON = JSON.stringify(data);

            // jQuery AJAX
            $.ajax({
            type: "POST",
            url: imageURL,
            data: dataJSON,
            error: function (e) {
                console.log(e);
                alert("ERROR: " + e.statusText + "\\nHTTP Status: " + e.status + "\\n\\nData not saved to database!");
            },
            dataType: "json",
            contentType: "application/json",
            });

            console.log(
            "Saved tag " + taggedAs + " for " + currentImage + " to database."
            );

        }

        $(document).keypress(function (e) {
            // keyCode values for the following characters:
            // 1 = 49
            // 2 = 50
            // 3 = 51
            // keyCode lets you check for the *character* pressed, so Numpad 1
            // is also keyCode 49.
            if (e.keyCode == 49) {
            console.log(
                "Handler for keypress fired with keyCode: " + e.keyCode + " (MALIGNANT)"
            );
            postToDB(1);
            getNewImage();
            } else if (e.keyCode == 50) {
            console.log(
                "Handler for keypress fired with keyCode: " +
                e.keyCode +
                " (INCONCLUSIVE)"
            );
            postToDB(2);
            getNewImage();
            } else if (e.keyCode == 51) {
            console.log(
                "Handler for keypress fired with keyCode: " + e.keyCode + " (BENIGN)"
            );
            postToDB(3);
            getNewImage();
            }
        });

        $(document).ready(function () {
            $("#100pc").click(function () {
            document.getElementById("image").width = "224";
            });

            $("#150pc").click(function () {
            // jQuery styleee
            $("#image")[0].width = "336";
            });

            $("#200pc").click(function () {
            document.getElementById("image").width = "448";
            });

            $("#250pc").click(function () {
            document.getElementById("image").width = "560";
            });

            $("#malignant").click(function () {
            postToDB(1);
            getNewImage();
            });

            $("#inconclusive").click(function () {
            postToDB(2);
            getNewImage();
            });

            $("#benign").click(function () {
            postToDB(3);
            getNewImage();
            });



            /*
            // Set the loss
            $.ajax({
            url: getLatestLossURL,
            beforeSend: function (xhr) {
                xhr.overrideMimeType("text/plain; charset=utf-8");
            },
            }).done(function (jsonData) {
            jsonData = JSON.parse(jsonData);

            console.log("jsonData.data is: " + jsonData.data);

            if (jsonData.data.length == 0) {
                document.getElementById("loss-text").innerHTML =
                "Loss: Loss appears after first image is tagged";
            } else {
                document.getElementById("loss-text").innerHTML =
                "Loss: " + jsonData.loss;
            }
            });


            // Set the accuracy text string and progress bar, if a test has been performed.
            $.ajax({
            url: getMetricsURL,
            beforeSend: function (xhr) {
                xhr.overrideMimeType("text/plain; charset=utf-8");
            },
            }).done(function (jsonData) {
            jsonData = JSON.parse(jsonData);

            if (jsonData.data.length == 0) {
                document.getElementById("accuracy-text").innerHTML =
                "Accuracy: No tests yet performed.";
            } else {
                document.getElementById("accuracy-text").innerHTML =
                "Accuracy: " + jsonData.data[jsonData.data.length - 1];
            }
            });
            */


        });
        </script>
        {%% endblock %%} {%% block title %%}Labeller{%% endblock %%} {%%
        block content %%} {%% include "navbar.html" %%}

        <div class="container">
        <h1 class="page-header">
            Labeller <small>Image tagging web application</small>
        </h1>
        <div class="row">
            <div class="col-md-4">
            <div class="panel panel-primary" style="background: whitesmoke">
                <div class="panel-heading">Infobox</div>
                <div class="panel-body">
                <h4>Image Details</h4>
                <p id="imageText">
                    Tile {{ r }} of {{ image_count }} ({{ rand_image }})
                </p>
                <hr />

                <!--
                <h4>Zoom Level</h4>
                <div class="list-group">
                    <a id="100pcAlt" href="#" class="list-group-item">100%%</a>
                    <a id="150pcAlt" href="#" class="list-group-item">150%%</a>
                    <a id="200pcAlt" href="#" class="list-group-item">200%%</a>
                </div>
                <hr />
                -->

                <h4>Zoom Level:</h4>

                <center>
                    <div class="btn-group btn-group-toggle" data-toggle="buttons">
                    <label id="100pc" class="btn btn-secondary active">
                        <input
                        type="radio"
                        name="options"
                        id="option1"
                        autocomplete="off"
                        checked
                        />
                        100%%
                    </label>
                    <label id="150pc" class="btn btn-secondary">
                        <input
                        type="radio"
                        name="options"
                        id="option2"
                        autocomplete="off"
                        />
                        150%%
                    </label>
                    <label id="200pc" class="btn btn-secondary">
                        <input
                        type="radio"
                        name="options"
                        id="option3"
                        autocomplete="off"
                        />
                        200%%
                    </label>
                    <label id="250pc" class="btn btn-secondary">
                        <input
                        type="radio"
                        name="options"
                        id="option3"
                        autocomplete="off"
                        />
                        250%%
                    </label>
                    </div>
                </center>



                <!--
                <hr />
                <div class="progress">
                    <div
                    class="progress-bar progress-bar-striped"
                    style="width: 0%%"
                    id="accuracy-progress-bar"
                    ></div>
                </div>

                -->

                <hr />
                <h4>Keyboard Shortcuts:</h4>
                <ul>
                    %s
                </ul>
                </div>
            </div>
            </div>
            <div class="col-md-8" align="center">
            <img id="image" width="224px" src="{{ rand_image }}" />
            <hr style="padding: 50px" />
                <div class="btn-group" role="group" aria-label="Basic example">
                %s
                </div>
            </div>
        </div>

        <!---
                <div class="row">
                <div class="col-md-12">
                    <hr>
                    <h3>Keybord shortcuts:</h2>
                    <p>Use <kbd>1</kbd> for <b>Malignant</b>, <kbd>2</kbd> for <b>Inconclusive</b>, and <kbd>3</kbd> for <b>Benign</b>.</p>
                </div>
                </div>
        -->
        </div>

        {%% include "footer.html" %%} {%% endblock %%}
        """ % (' '.join(keyboard_shortcuts), ' '.join(buttons))


#!/usr/bin/env python

from setuptools import setup
import os

with open('PyPI.md') as f:
    long_description = f.read()

setup(name='labeller',
        packages=['labeller'],
        version='0.1.4',
        description='Create image labelling software for machine learning applications.',
        long_description=long_description,
        long_description_content_type='text/markdown',
        license='MIT',
        author='Marcus D. Bloice',
        author_email='marcus.bloice@medunigraz.at',
        url='https://github.com/mdbloice/Labeller',
        classifiers=[
        'Development Status :: 4 - Beta',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        ],
        install_requires=[
        'flask>=2.0',
        'flask-bootstrap>=3.3'
        ],
        # python_requires='>=3.5',
        # include_package_data=True,  # For the MANIFEST.in file...
        package_data={'': ['resources/*']}
        #scripts=['capitalize/bin/cap_script.py'],
    )



# labellerExec.py
# Author: Marcus D. Bloice <https://github.com/mdbloice>
# Licensed under the terms of the MIT Licence.
import argparse
import sys
import sqlite3
import os
from collections import OrderedDict
import glob
import pkg_resources
import pickle

# Internal imports
from labeller.htmlElements import Footer, Navbar, Index

parser = argparse.ArgumentParser(description='Generate an image labelling \
    application.', prog='python -m labeller')
parser.add_argument('class_names', metavar='class_names', nargs='+',
    help='a list of class names seperated by spaces')

args = parser.parse_args()

# Get the number of classes provided
n_classes = len(args.class_names)

if n_classes <= 1:
    print('Minimum number of classes is 2, you provided %s: %s' % (n_classes,
        args.class_names))
    sys.exit(1)

# Arguments look good.
print("Attempting to generate application with %s classes: %s." % (n_classes,
    ', '.join(args.class_names)))

extensions = ["*.png", "*.PNG", "*.jpg", "*.JPG", "*.jpeg", "*.JPEG"]

im_dir = os.path.join('static', 'images')

if not os.path.exists(os.path.join('.', im_dir)):
    print("No directory named '%s' found in the current directory: %s). Aborting." % (im_dir, os.getcwd()))
    sys.exit(1)
else:
    print("Found a directory named %s: %s\nSearching for images with the following extensions: %s" % (im_dir, os.path.join(os.getcwd(), im_dir), ' '.join(extensions)))

image_paths = []
for ext in extensions:
    image_paths.extend(glob.glob(os.path.join('.', im_dir, ext)))

if len(image_paths) <= 0:
    print("No images found in %s. Aborting." % os.path.join(os.getcwd(), im_dir))
    sys.exit(1)
else:
    print("Found %s images in %s" %(len(image_paths), os.path.join(os.getcwd(), im_dir)))

# Create a config file as a pickle that we read later when the app is run.
# NOT CURRENTLY NEEDED
#with open(os.path.join('.', 'labeller.pkl'), 'wb') as to_write:
#    pickle.dump(args.class_names, to_write)

print("Generating application with %s classes with the labels %s for %s images." % (n_classes, ', '.join(args.class_names), len(image_paths)))

# The first 9 classes get keyboard shortcuts from 1-9
# Do this in the appropriate class in htmlElements.py
#classes = OrderedDict()
#classes = {args.class_names[x]:x+1 for x in range(len(args.class_names))}

# Create a database using the class labels provided
os.makedirs(os.path.join('.', 'db'), exist_ok=True)
conn = sqlite3.connect(os.path.join('.', 'db', 'labels.db'))
conn.execute('CREATE TABLE IF NOT EXISTS labels (id INTEGER PRIMARY KEY, image STRING, label INTEGER, label_string STRING)')

# Insert some test data: Not required for release.
#cur = conn.cursor()
#cur.execute('INSERT INTO labels (image, label, label_string) VALUES("test.png", 0, "cat")')
#cur.execute('INSERT INTO labels (image, label, label_string) VALUES("cat.jpg", 0, "cat");')
#cur.execute('INSERT INTO labels (image, label, label_string) VALUES("xyz.png", 1, "dog");')
#cur.execute('INSERT INTO labels (image, label, label_string) VALUES("xyz.png", 1, "dog");')
#cur.execute('INSERT INTO labels (image, label, label_string) VALUES("kitty.JPEG", 0, "cat");')
#conn.commit()
#cur.close()

conn.close()

# Create directory structure
os.makedirs(os.path.join('.', 'templates'), exist_ok=True)
os.makedirs(os.path.join('.', 'static', 'styles'), exist_ok=True)
# Places images in here for now, fix later using 'send_from_directory'
os.makedirs(os.path.join('.', 'static', 'images'), exist_ok=True)

# Use pkg_resources to open files or data installed with the package
app_py = pkg_resources.resource_string(__name__, os.path.join('resources', 'app.py'))
dashboard_css = pkg_resources.resource_string(__name__, os.path.join('resources', 'dashboard.css'))
about_html = pkg_resources.resource_string(__name__, os.path.join('resources', 'about.html'))
labels_html = pkg_resources.resource_string(__name__, os.path.join('resources', 'labels.html'))
navbar_html = pkg_resources.resource_string(__name__, os.path.join('resources', 'navbar.html'))
favicon_ico = pkg_resources.resource_string(__name__, os.path.join('resources', 'favicon.ico'))
# bootstrap_min_css = pkg_resources.resource_string(__name__, os.path.join('resources', 'bootstrap.min.css'))

# Copy any static files to their appropriate directories here
with open(os.path.join('.', 'app.py'), 'wb') as to_write:
    to_write.write(app_py)

with open(os.path.join('.', 'static', 'styles', 'dashboard.css'), 'wb') as to_write:
    to_write.write(dashboard_css)

with open(os.path.join('.', 'static', 'favicon.ico'), 'wb') as to_write:
    to_write.write(favicon_ico)

with open(os.path.join('.', 'templates', 'about.html'), 'wb') as to_write:
    to_write.write(about_html)

with open(os.path.join('.', 'templates', 'labels.html'), 'wb') as to_write:
    to_write.write(labels_html)

with open(os.path.join('.', 'templates', 'navbar.html'), 'wb') as to_write:
    to_write.write(navbar_html)

# Currently CDNs are used for the Bootstrap CSS, but this may change in future
#with open(os.path.join('.', 'static', 'bootstrap.min.css'), 'wb') as to_write:
#    to_write.write(bootstrap_min_css)

# Create any dynamic content here.
footer = Footer()
with open(os.path.join('.', 'templates', 'footer.html'), 'w') as to_write:
    to_write.write(footer.get_html())

index = Index(class_names=args.class_names)
with open(os.path.join('.', 'templates', 'index.html'), 'w') as to_write:
    to_write.write(index.get_html())

# Navbar currently is not dynamic content and can be copied from the static
# package resources
#navbar = Navbar()
#with open(os.path.join('.', 'templates', 'navbar.html'), 'w') as to_write:
#    to_write.write(navbar.get_html())

# See https://docs.python.org/3/library/__main__.html#module-__main__
# and https://docs.python.org/3/using/cmdline.html#cmdoption-m
# if __name__ == "__main__":
#     print("This is from labellerExec.py")
